import tkinter as tk
import tkinter.filedialog as fd
import pandas as pd
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb

def askinfilename(msg = None, types = [('', '*.*')]):
    """
            入力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.askopenfilename(title = msg, filetypes = types)
    rt.destroy()
    return flnm

def askoutfilename(msg = None, types = [('', '*.*')], deftype = '.txt'):
    """
            出力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.asksaveasfilename(title = msg, filetypes = types,
                                           defaultextension = deftype)
    rt.destroy()
    return flnm

finnm = askinfilename('入力データExcelファイル', types = [('Excel(*.xlsx)', '*.xlsx')])
foutnm = askoutfilename(msg = '画像保存ファイル', types = [('画像 (*.png)', '*.png')],
                       deftype = '.png')
ftxtoutnm = askoutfilename(msg = 'テキスト出力ファイル', types = [('Text (*.txt)', '*.txt')],
                           deftype = '.txt')

fout = open(ftxtoutnm, 'w')
fout.write('入力データファイル = {}\n'.format(finnm))

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

K = len(data[0]) - 1
print('K = ', K)
fout.write('\nK = {}\n'.format(K))
VarGrp = []
for i in range(1, K + 1):
    VarGrp.append(data[0][i])
print(VarGrp)
fout.write('\n{}\n'.format(VarGrp))

VarCnd1 = data[1][0]
VarCnd2 = data[2][0]
print(VarCnd1, ', ', VarCnd2)
fout.write('\n{0:},  {1:}\n'.format(VarCnd1, VarCnd2))

Data = np.empty((K, 2), dtype = int)
for k in range(K):
    for i in range(2):
        Data[k][i] = data[i + 1][k + 1]

print(Data)
fout.write('\n{}\n'.format(Data))

StanData = {'K': K, 'Data': Data}

sm = pystan.StanModel(file = 'BinModel.stan')
fit = sm.sampling(data = StanData, n_jobs = 1)
print(fit)
fout.write('\n{}\n'.format(fit))


#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

plt.figure(figsize = (9, 5))
plt.title('pの事後分布', fontsize = 18)
for k in range(K):
    sb.kdeplot(fit['p'][:, k], label = VarGrp[k])
plt.yticks([])
plt.xlabel('p', fontsize = 14)
plt.legend(loc = 'upper right', bbox_to_anchor = (1.25, 1.0), fontsize = 12)
plt.tight_layout()
plt.savefig(foutnm)
plt.show()

print('\n\nP({})\n'.format(VarCnd2))
fout.write('\n\n\nP({})\n\n'.format(VarCnd2))
for k in range(K):
    print('\n{}:'.format(VarGrp[k]))
    fout.write('\n{}:\n'.format(VarGrp[k]))
    p_mean = fit['p'][:, k].mean()
    p_025, Q1, Med, Q3, p_975 = np.percentile(fit['p'][:, k], [2.5, 25, 50, 75, 97.5])
    print('平均   = {0:.5f}'.format(p_mean))
    fout.write('平均   = {0:.5f}\n'.format(p_mean))
    print('中央値 = {0:.5f}'.format(Med))
    fout.write('中央値 = {0:.5f}\n'.format(Med))
    print('Q1 = {0:.5f}     Q3 = {1:.5f}'.format(Q1, Q3))
    fout.write('Q1 = {0:.5f}     Q3 = {1:.5f}\n'.format(Q1, Q3))
    print('95%確信区間 = [{0:.5f}, {1:.5f}]'.format(p_025, p_975))
    fout.write('95%確信区間 = [{0:.5f}, {1:.5f}]\n'.format(p_025, p_975))

fout.close()

